package com.liangjian.search.controller;

import com.liangjian.search.entity.QAProblem;
import com.liangjian.search.entity.response.ResultObject;
import com.liangjian.search.es.QAProblemEs;
import com.liangjian.search.kafka.QAProblemJob;
import com.liangjian.search.service.IQAProblemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.SendResult;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.concurrent.TimeUnit;

/**
 * Created by huangziqi on 2020/1/3
 */
@RestController
@RequestMapping("/qa")
@Api(value = "问答搜索控制器",description = "问答搜索控制器", tags = "问答搜索控制器")
public class QAProblemController {

    @Autowired
    private IQAProblemService iqaProblemService;
    @Autowired
    private KafkaTemplate<Object, QAProblemJob> kafkaTemplate;

    @GetMapping("/search")
    @ApiOperation(value = "搜索问题")
    public ResultObject<Page<QAProblemEs>> search(@ApiParam(value = "搜索关键词",required = true) @RequestParam String kw,
                                                  @ApiParam(value = "页数",required = true) @RequestParam Integer page,
                                                  @ApiParam(value = "每页数量",required = true) @RequestParam Integer size) {
        if (page * size > 100000) {
            return ResultObject.error("无法查询10万后的数据");
        } else {
            return ResultObject.success(iqaProblemService.search(kw,page-1,size));
        }
    }

    @GetMapping("/think")
    @ApiOperation(value = "搜索联想")
    public List<String> think(@ApiParam(value = "搜索关键词",required = true) @RequestParam String kw) {
        return iqaProblemService.think(kw);
    }

    @PostMapping("/problem/{id}")
    @ApiOperation(value = "新增问题")
    public ResultObject<Object> addProblem(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
//        if (id == null) {
//            return ResultObject.error("没有提供问题id");
//        } else if (!iqaProblemService.addQAProblemEs(id)) {
//            return ResultObject.error("保存失败");
//        } else {
//            return ResultObject.success("保存成功",null);
//        }
        if (id == null) {
            return ResultObject.error("没有提供问题id");
        } else {
            try {
                kafkaTemplate.send("qa_problem_job", QAProblemJob.Create(id)).get(60, TimeUnit.SECONDS);
                return ResultObject.success("提交成功",null);
            } catch (Exception ex) {
                ex.printStackTrace();
                return ResultObject.error("发生错误。");
            }
        }
    }

    @PutMapping("/problem/{id}")
    @ApiOperation(value = "更新问题")
    public ResultObject<Object> editProblem(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
//        if (id == null) {
//            return ResultObject.error("没有提供问题id");
//        } else if (!iqaProblemService.editQAProblemEs(id)) {
//            return ResultObject.error("保存失败");
//        } else {
//            return ResultObject.success("保存成功",null);
//        }
        if (id == null) {
            return ResultObject.error("没有提供问题id");
        } else {
            try {
                kafkaTemplate.send("qa_problem_job", QAProblemJob.Update(id)).get(60, TimeUnit.SECONDS);
                return ResultObject.success("提交成功",null);
            } catch (Exception ex) {
                ex.printStackTrace();
                return ResultObject.error("发生错误。");
            }
        }
    }

    @DeleteMapping("/problem/{id}")
    @ApiOperation(value = "删除问题")
    public ResultObject<Object> deleteProblem(@ApiParam(value = "id", required = true) @PathVariable("id") Long id) {
//        if (id == null) {
//            return ResultObject.error("没有提供问题id");
//        } else if (!iqaProblemService.deleteQAProblemEs(id)) {
//            return ResultObject.error("删除失败");
//        } else {
//            return ResultObject.success("删除成功",null);
//        }
        if (id == null) {
            return ResultObject.error("没有提供问题id");
        } else {
            try {
                kafkaTemplate.send("qa_problem_job", QAProblemJob.Delete(id)).get(60, TimeUnit.SECONDS);
                return ResultObject.success("提交成功",null);
            } catch (Exception ex) {
                ex.printStackTrace();
                return ResultObject.error("发生错误。");
            }
        }
    }
}
