package com.liangjian.search.controller;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.liangjian.search.entity.QAProblemJobFailedMsg;
import com.liangjian.search.entity.response.ResultObject;
import com.liangjian.search.kafka.QAProblemJob;
import com.liangjian.search.service.IQAProblemJobFailedMsgService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.concurrent.TimeUnit;

/**
 * Created by huangziqi on 2020/1/10
 */
@RestController
@RequestMapping("/qa/error")
@Api(value = "问答搜索的保存ES失败任务管理器", tags = "问答搜索的保存ES失败任务管理器")
public class QAProblemJobController {
    @Autowired
    private IQAProblemJobFailedMsgService iqaProblemJobFailedMsgService;
    @Autowired
    private KafkaTemplate<Object,QAProblemJob> kafkaTemplate;

    @GetMapping("/manager")
    public IPage<QAProblemJobFailedMsg> getMsgs(@ApiParam(value = "page",required = true) @RequestParam Integer page,
                                               @ApiParam(value = "size",required = true) @RequestParam Integer size) {
        IPage<QAProblemJobFailedMsg> pager = new Page<>(page,size);
        QueryWrapper<QAProblemJobFailedMsg> wrapper = new QueryWrapper<>();
        wrapper.orderByDesc("CreateDate");
        return iqaProblemJobFailedMsgService.page(pager,wrapper);
    }

    @GetMapping("/manager/retry/{id}")
    public ResultObject<Object> retry(@ApiParam(value = "id",required = true) @PathVariable Integer id) {
        QAProblemJobFailedMsg msg = iqaProblemJobFailedMsgService.getById(id);
        if(msg == null) {
            return ResultObject.error("查询不到id，请重新获取");
        }

        //构造拼接job
        QAProblemJob job = new QAProblemJob();
        job.setCrudType(msg.getCrudType());
        job.setId(msg.getDataId());

        //重新丢回kafka中
        try {
            kafkaTemplate.send("qa_problem_job_error", job).get(60, TimeUnit.SECONDS);
            //重试时直接删除数据库中的记录
            iqaProblemJobFailedMsgService.getBaseMapper().deleteById(id);
            return ResultObject.success("提交重试成功",null);
        } catch (Exception ex) {
            ex.printStackTrace();
            return ResultObject.error("提交重试失败");
        }
    }
}
