package com.liangjian.search.service;

import com.liangjian.search.entity.QAProblemJobFailedMsg;
import com.liangjian.search.kafka.QAProblemJob;
import com.liangjian.search.util.Tuple;
import com.liangjian.search.util.Tuple2;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 * Created by huangziqi on 2020/1/9
 */
@Service
public class QAProblemJobListener {
    private final Logger logger = LoggerFactory.getLogger(QAProblemJobListener.class);

    @Autowired
    private IQAProblemService iqaProblemService;
    @Autowired
    private IQAProblemJobFailedMsgService iqaProblemJobFailedMsgService;
    @Autowired
    private KafkaTemplate<Object,QAProblemJob> kafkaTemplate;

    //正常队列
    @KafkaListener(id = "qa_problem_consumer_group_test", topics = "qa_problem_job_test")
    public void consumeJob(ConsumerRecord<Object,QAProblemJob> record, Acknowledgment ack) {
        ack.acknowledge(); //无论是否成功都直接提交
        var job = record.value();
        logger.info("receive job : {}",job);
        Tuple2<Integer,String> res = Tuple.of(IQAProblemService.FAILED,"unknown error");
        switch (job.getCrudType()) {
            case Create: res = iqaProblemService.addQAProblemEs(job.getId()); break;
            case Update: res = iqaProblemService.editQAProblemEs(job.getId()); break;
            case Delete: res = iqaProblemService.deleteQAProblemEs(job.getId()); break;
//            default: logger.error("wrong job type. {}",job);
        }

        if (res.get_1() == IQAProblemService.SUCCESS) {
            //成功消费
            logger.info("consumed job : {}",job);
        } else if(res.get_1() == IQAProblemService.FAILED) {
            //消费失败，放到死心队列，疯狂循环消费
            kafkaTemplate.send("qa_problem_job_error",job);
        } else if(res.get_1() == IQAProblemService.NO_SUCH_ID) {
            //数据库中，或者更新时ES中没有这个id，忽略这条记录
            logger.warn("no such id, pass job : {}",job);
        }
    }

    //正常队列错误的的死信对列
    @KafkaListener(id = "qa_problem_error_consumer_group_test", topics = "qa_problem_job_error_test")
    public void consumeErrorJob(ConsumerRecord<Object,QAProblemJob> record, Acknowledgment ack) {
        var job = record.value();
        logger.info("[error queue] receive job : {}",job);

        Tuple2<Integer,String> res = Tuple.of(IQAProblemService.FAILED,"unknown error");
        //重试3次，不行就算
        for (var i = 0; i < 3; i++) {
            //与正常的一样的逻辑
            switch (job.getCrudType()) {
                case Create: res = iqaProblemService.addQAProblemEs(job.getId());break;
                case Update: res = iqaProblemService.editQAProblemEs(job.getId());break;
                case Delete: res = iqaProblemService.deleteQAProblemEs(job.getId());break;
//            default: logger.error("[error queue] wrong job type. {}",job);
            }

            if (res.get_1() == IQAProblemService.SUCCESS) {
                //成功消费
                logger.info("[error queue] consumed job : {}", job);
                break;
            } else if (res.get_1() == IQAProblemService.FAILED) {
                //消费失败，放到死心队列，疯狂循环消费
                logger.error("[error queue] consume error , to continue : {}", job);
            } else if (res.get_1() == IQAProblemService.NO_SUCH_ID) {
                //数据库中，或者更新时ES中没有这个id，忽略这条记录
                logger.warn("[error queue] no such id, pass job : {}", job);
                break;
            }
        }

        ack.acknowledge();
        //若还是错，就直接移进数据库，后期管理
        if (res.get_1() == IQAProblemService.FAILED) {
            QAProblemJobFailedMsg msg = new QAProblemJobFailedMsg();
            msg.setCrudType(job.getCrudType());
            msg.setDataId(job.getId());
            msg.setMsg(res.get_2());
            msg.setCreateDate(new Date());
            iqaProblemJobFailedMsgService.save(msg);
        }
    }
}
