package com.liangjian.search.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.liangjian.search.dao.QAProblemEsRepository;
import com.liangjian.search.dao.QAProblemMapper;
import com.liangjian.search.entity.QAProblem;
import com.liangjian.search.entity.QAProblemTag;
import com.liangjian.search.entity.param.IdOffsetLimit;
import com.liangjian.search.es.QAProblemEs;
import com.liangjian.search.service.IQAProblemService;
import com.liangjian.search.service.IQAProblemTagService;
import com.liangjian.search.util.Tuple;
import com.liangjian.search.util.Tuple2;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.search.builder.SearchSourceBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.elasticsearch.core.ElasticsearchRestTemplate;
import org.springframework.data.elasticsearch.core.query.NativeSearchQuery;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

import static org.elasticsearch.index.query.QueryBuilders.*;

/**
 * Created by huangziqi on 2020/1/3
 */
@Service
public class QAProblemService extends ServiceImpl<QAProblemMapper,QAProblem> implements IQAProblemService {
    @Autowired
    private QAProblemEsRepository qaProblemEsRepository;
    @Autowired
    private IQAProblemTagService iqaProblemTagService;
    @Autowired
    private ElasticsearchRestTemplate elasticsearchRestTemplate;

    @Override
    public List<QAProblem> getWrappedProblem(Long id, Integer offset, Integer limit) {
        return getBaseMapper().getWrappedProblem(IdOffsetLimit.of(id, offset, limit));
    }

    @Override
    public Page<QAProblemEs> search(String keyword, Integer page, Integer size) {
        Pageable pageable = PageRequest.of(page,size);
        BoolQueryBuilder queryBuilder = boolQuery().
                must(termQuery("state", 1)).
                must(boolQuery().
                        should(matchQuery("tags", keyword).boost(10)).
//                        should(matchQuery("tags.pinyin", keyword).boost(90)).
                        should(matchQuery("title", keyword).boost(5))
//                        should(matchQuery("title.pinyin", keyword).boost(9)).
//                        should(matchQuery("describe", keyword)).
//                        should(matchQuery("describe.pinyin", keyword))
                );
        SearchSourceBuilder sourceBuilder = new SearchSourceBuilder().query(queryBuilder);
        SearchRequest request = new SearchRequest("qa_problem").source(sourceBuilder);
        try {
            SearchResponse response = elasticsearchRestTemplate.getClient().search(request, RequestOptions.DEFAULT);
            Float maxScore = response.getHits().getMaxScore();
            Double mustMinScore = (maxScore * 0.0);

            var secondSearch = new NativeSearchQuery(queryBuilder);
            secondSearch.setMinScore(mustMinScore.floatValue());
            secondSearch.setPageable(pageable);
            var pager = elasticsearchRestTemplate.queryForPage(secondSearch,QAProblemEs.class);
            return pager;
        } catch (Exception ex) {
           ex.printStackTrace();
           var pager = qaProblemEsRepository.search(sourceBuilder.query(),pageable);
           return pager;
        }
        //return qaProblemEsRepository.search(sourceBuilder.query(),pageable);
    }

    @Override
    public List<String> think(String keyword) {
        boolean isChinese = keyword.getBytes().length != keyword.length();
        BoolQueryBuilder queryBuilder = boolQuery().must(termQuery("state", 1));
        if(isChinese) {
            queryBuilder = queryBuilder.must(boolQuery().
                    should(matchQuery("tags", keyword).boost(10)).
                    should(matchQuery("title", keyword).boost(5))
            );
        } else {
            queryBuilder = queryBuilder.must(boolQuery().
                    should(matchQuery("tags.pinyin", keyword).boost(10)).
                    should(matchQuery("title.pinyin", keyword).boost(5))
            );
        }
        Page<QAProblemEs> page = qaProblemEsRepository.search(queryBuilder, PageRequest.of(0,10));
        Set<String> set = page.getContent().stream()
                .map(i -> i.getTags() == null ? i.getTitle() : i.getTags())
                .flatMap(i -> Arrays.stream(i.split("\\|")))
                .collect(Collectors.toSet());
        List<String> res =  new ArrayList<>(set);
        res = res.stream()
                .sorted(Comparator.comparingInt(String::length))
                .collect(Collectors.toList());
        return res;
    }

    @Override
    public Tuple2<Integer,String> addQAProblemEs(Long id) {
        var list = getWrappedProblem(id,0,1);
        if(list.isEmpty()) {
            return Tuple.of(NO_SUCH_ID,"数据库中没有这个id");
        } else {
            var problem = list.get(0);
            var tagList = iqaProblemTagService
                    .getBaseMapper()
                    .selectList(
                            new QueryWrapper<QAProblemTag>().eq("ProblemId",problem.getId()));
            var tags = tagList.stream().map(i -> i.getTagName()).reduce((i,j) -> i+"|"+j).orElse("");
            try {
                ObjectMapper objectMapper = new ObjectMapper();
                var json = objectMapper.writerFor(QAProblem.class).writeValueAsString(problem);
                QAProblemEs problemEs = objectMapper.readerFor(QAProblemEs.class).readValue(json);
                problemEs.setTags(tags);
                qaProblemEsRepository.save(problemEs);
                return Tuple.of(SUCCESS,"");
            } catch (Exception ex) {
                ex.printStackTrace();
                return Tuple.of(FAILED, ex.getMessage());
            }
        }
    }

    @Deprecated
    @Override
    public Tuple2<Boolean,String> addQAProblemESBatch(List<QAProblemEs> list) {
        try {
            qaProblemEsRepository.saveAll(list);
            return Tuple.of(true,"");
        } catch (Exception ex) {
            ex.printStackTrace();
            return Tuple.of(false,ex.getMessage());
        }
    }

    @Override
    public Tuple2<Integer,String> editQAProblemEs(Long id) {
        var list = getWrappedProblem(id,0,1);
        if(list.isEmpty()) {
            return Tuple.of(NO_SUCH_ID,"数据库中没有这个id");
        } else {
            var problem = list.get(0);
            var tagList = iqaProblemTagService
                    .getBaseMapper()
                    .selectList(
                            new QueryWrapper<QAProblemTag>().eq("ProblemId",problem.getId()));
            var tags = tagList.stream().map(i -> i.getTagName()).reduce((i,j) -> i+"|"+j).orElse("");
            try {
                var opt = qaProblemEsRepository.findById(id);
                if (opt.isEmpty()) {
                    return Tuple.of(NO_SUCH_ID,"ES中没有这个id");
                }
                var problemEs = opt.get();
                problemEs.setTitle(problem.getTitle());
                problemEs.setDescribe(problem.getDescribe());
                problemEs.setCategoryCode(problem.getCategoryCode());
                problemEs.setOriginType(problem.getOriginType());
                problemEs.setState(problem.getState());
                problemEs.setAnonymous(problem.getAnonymous());
                problemEs.setRecommend(problem.getRecommend());
                problemEs.setCreateTime(problem.getCreateTime());
                problemEs.setDoctorId(problem.getDoctorId());
                problemEs.setUserId(problem.getUserId());
                problemEs.setUpdateTime(problem.getUpdateTime());
                problemEs.setUpdateName(problem.getUpdateName());
                problemEs.setUpdateId(problem.getUpdateId());
                problemEs.setSolve(problem.getSolve());
                problemEs.setBrowseSum(problem.getBrowseSum());
                problemEs.setAge(problem.getAge());
                problemEs.setSex(problem.getSex());
                problemEs.setReleaseTime(problem.getReleaseTime());
                problemEs.setQaId(problem.getQaId());
                problemEs.setMobilePhone(problem.getMobilePhone());
                problemEs.setReplyCount(problem.getReplyCount());
                problemEs.setReplyerName(problem.getReplyerName());
                problemEs.setLastReply(problem.getLastReply());
                problemEs.setContent(problem.getContent());
                problemEs.setTags(tags);
                qaProblemEsRepository.save(problemEs);
                return Tuple.of(SUCCESS,"");
            } catch (Exception ex) {
                ex.printStackTrace();
                return Tuple.of(FAILED,ex.getMessage());
            }
        }
    }

    @Override
    public Tuple2<Integer,String> deleteQAProblemEs(Long id) {
        try {
            qaProblemEsRepository.deleteById(id);
            return Tuple.of(SUCCESS,"");
        } catch (Exception ex) {
            ex.printStackTrace();
            return Tuple.of(FAILED,ex.getMessage());
        }
    }
}
